﻿using BPA.Common.Enums.Queue;
using BPA.Component.RabbitMQClient.Base;
using BPA.SaaS.BizLog.Api.IRepository;
using BPA.SaaS.BizLog.Api.IService.Queue;
using Microsoft.Extensions.Logging;
using BPA.Common.Infrastructure.BizLogs;
using BPA.Component.Extensions;
using BPA.SaaS.BizLog.Api.Entity;
using RabbitMQ.Client.Events;

namespace BPA.SaaS.BizLog.Api.Service.Queue;

/// <summary>
/// 业务日志消费者端
/// </summary>
public class BizLogConsumerService : BaseConsumer<BaseBizLogMsgBody, BizLogConsumerService>, IBizLogConsumerService
{
    private readonly IServiceProvider _serviceProvider;
    private readonly ILogger<BizLogConsumerService> _logger;
    private readonly IBizLogRepository _bizLogRepository;
    private readonly IBizLogTimeoutRepository _bizLogTimeoutRepository;

    /// <summary>
    /// 构造方法
    /// </summary>
    /// <param name="serviceProvider"></param>
    /// <param name="logger"></param>
    /// <param name="bizLogRepository"></param>
    /// <param name="bizLogTimeoutRepository"></param>
    public BizLogConsumerService(IServiceProvider serviceProvider, ILogger<BizLogConsumerService> logger,
        IBizLogRepository bizLogRepository,
        IBizLogTimeoutRepository bizLogTimeoutRepository) :
        base(logger)
    {
        _serviceProvider = serviceProvider;
        _logger = logger;
        _bizLogRepository = bizLogRepository;
        _bizLogTimeoutRepository = bizLogTimeoutRepository;
    }

    /// <summary>
    /// 消费出队
    /// </summary>
    /// <param name="msgBody"></param>
    /// <param name="eventArgs"></param>
    /// <returns></returns>
    public override bool Dequeue(BaseBizLogMsgBody msgBody, BasicDeliverEventArgs eventArgs)
    {
        _logger.LogDebug("Received message:");
        _logger.LogDebug($"{msgBody.ToJson()}");
        _logger.LogDebug(string.Empty);

        return msgBody.EnumMessageType switch
        {
            EnumMessageType.BizLog => AddBizLogs(msgBody),
            EnumMessageType.BizLogExt => AddBizLogExt(msgBody),
            _ => false
        };
    }

    private bool AddBizLogExt(BaseBizLogMsgBody msgBody) =>
        _bizLogTimeoutRepository.AddTimeoutBizLog(msgBody.MapTo<BPA_BizLog_Execetion_Time, BaseBizLogMsgBody>());

    private bool AddBizLogs(BaseBizLogMsgBody msgBody)
    {
        if (msgBody.CreateDateTime.HasValue && msgBody.CreateBy != null)
        {
            var bpaBizLogCreate = msgBody.MapTo<BPA_BizLog_Create, BaseBizLogMsgBody>();
            if (msgBody.EnumBizLogType != null) bpaBizLogCreate.EnumBizLogType = msgBody.EnumBizLogType.Value;
            return _bizLogRepository.AddCreateBizLog(bpaBizLogCreate);
        }

        if (msgBody.UpdateDateTime.HasValue && msgBody.UpdateBy != null)
        {
            var bpaBizLogCreate = msgBody.MapTo<BPA_BizLog_Update, BaseBizLogMsgBody>();
            if (msgBody.EnumBizLogType != null) bpaBizLogCreate.EnumBizLogType = msgBody.EnumBizLogType.Value;
            return _bizLogRepository.AddUpdateBizLog(bpaBizLogCreate);
        }

        if (msgBody.DeleteDateTime.HasValue && msgBody.DeleteBy != null)
        {
            var bpaBizLogDelete = msgBody.MapTo<BPA_BizLog_Delete, BaseBizLogMsgBody>();
            if (msgBody.EnumBizLogType != null) bpaBizLogDelete.EnumBizLogType = msgBody.EnumBizLogType.Value;
            return _bizLogRepository.AddDeleteBizLog(bpaBizLogDelete);
        }

        if (!msgBody.QueryTime.HasValue || msgBody.QueryBy == null) return false;
        {
            var bpaBizLogQuery = msgBody.MapTo<BPA_BizLog_Query, BaseBizLogMsgBody>();
            if (msgBody.EnumBizLogType != null) bpaBizLogQuery.EnumBizLogType = msgBody.EnumBizLogType.Value;
            return _bizLogRepository.AddQueryBizLog(bpaBizLogQuery);
        }
    }
}